(async () => {
  const fadeDuration = 0.4;
  const volume = 0;
  const numFiles = 15;
  const basePath = "../../_media/";
  const movieTypes = ["webm", "mp4", "mov"];
  const imageTypes = ["png", "gif", "jpg", "jpeg", "svg"];
  const container = document.querySelector("#container");
  const imageDuration = 5;

    const fileTypes = [...movieTypes.map(type => ({ type, isVideo: true })),
                       ...imageTypes.map(type => ({ type, isImage: true }))];
  
    let loopers = [];
  
    class Looper {
      constructor(fileName) {
        this.fileName = fileName;
        this.isValid = false;
      }
  
      async load() {
        const baseUrl = basePath + this.fileName;
        
        for (const config of fileTypes) {
          const url = `${baseUrl}.${config.type}`;
          try {
            const response = await Promise.race([this.timeoutFetch(url), this.timer(100)]);
            if (response.ok) {
              this.url = url;
              this.isVideo = config.isVideo;
              this.isImage = config.isImage;
              const element = await this.createElement();
              if (element) {
                this.element = element;
                this.isValid = true;
              }
              break;
            }
          } catch (error) {
            console.error(`Error fetching file ${url}: ${error}`);
          }
        }
      }
      
      async timeoutFetch(url) {
        const response = await fetch(url);
        if (!response.ok) {
          throw new Error(`HTTP error! status: ${response.status}`);
        }
        return response;
      }
      
      timer(ms) {
        return new Promise((res, rej) => setTimeout(() => rej('Request Timeout'), ms));
      }
  
      fetchFile(config) {
        return fetch(config.url)
          .then(response => response.ok ? config : null)
          .catch(error => {
            console.error(`Error fetching file ${config.url}: ${error}`);
            return null;
          });
      }
  
      createImage() {
        return new Promise((resolve) => {
          const element = new Image();
          element.src = this.url;
          element.onload = fulfill;
          element.onerror = fulfill;
  
          function fulfill(e) {
            element.onload = null;
            element.onerror = null;
            if (e.type === "error") {
              resolve();
            } else {
              resolve(element);
            }
          }
        });
      }
  
      createVideo() {
        return new Promise((resolve) => {
          const element = document.createElement("video");
          element.src = this.url;
          element.volume = volume;
  
          if (element.readyState > 3) {
            resolve(element);
          } else {
            element.oncanplaythrough = fulfill;
            element.onerror = fulfill;
          }
  
          function fulfill(e) {
            element.oncanplaythrough = null;
            element.onerror = null;
            if (e.type === "error") {
              resolve();
            } else {
              resolve(element);
            }
          }
        });
      }
  
      createElement() {
        if (this.isVideo) {
          return this.createVideo();
        }
        return this.createImage();
      }
  
      play() {
        if (this.isVideo) {
          this.playVideo();
        } else {
          this.playImage();
        }
      }
  
      playNext() {
        if (!this.delay) {
          return this.next.play();
        }
  
        gsap.delayedCall(this.delay, () => this.next.play());
      }
  
      playVideo() {
        const element = this.element;
        element.autoplay = true;
        element.load();
        element.currentTime = 0;
        element.play();
        gsap.set(element, { autoAlpha: 1 });
        element.onended = () => {
          element.onended = null;
          gsap.set(element, { autoAlpha: 0 });
          this.playNext();
        }
      }
  
      playImage() {
        gsap.timeline({ onComplete: () => this.playNext()})
          .to(this.element, {
            duration: fadeDuration,
            autoAlpha: 1
          })
          .to(this.element, {
            duration: fadeDuration,
            autoAlpha: 0
          }, "+=" + (imageDuration - fadeDuration * 2));   
      }
    }
  
    init();
  
    async function init() {
      const looperPromises = Array.from({ length: numFiles }, async (_, i) => {
        const looper = new Looper(i + 1);
        await looper.load();
        return looper.isValid ? looper : null;
      });
  
      loopers = (await Promise.all(looperPromises)).filter(Boolean);
  
      if (!loopers.length) {
        return showWarning();
      }
  
      const wrap = gsap.utils.wrap(loopers);
  
      loopers.forEach((looper, i) => {
        container.append(looper.element);
        looper.next = wrap(i + 1);
  
        if (delayBetweenEach) {
          looper.delay = delay;
        }
      });
  
      if (!delayBetweenEach) {
        loopers[loopers.length - 1].delay = delay;
      }
  
      loopers[0].play();
  
      gsap.set(container, { autoAlpha: 1 });
    }
  
    function showWarning() {
      fetch("../../_code/warning.html")
        .then(response => response.text())
        .then(data => $("body").append(data))
        .catch(error => console.log(error));
    }
  })();
  